use super::helpers;
use wccg_models::activity::RiderMonthlySummary;
use wccg_models::activity::{ActivityType, Stats};
use wccg_models::any::Any;
use wread_data_mongodb::crud_repository;
use wread_data_mongodb::mongodb::{bson::doc, error::Error, Database};

const COLLECTION_NAME: &str = "riderMonthlySummary";

pub async fn get_by_rider_id(
    rider_id: &str,
    db: &Database,
) -> Result<Vec<RiderMonthlySummary>, Error> {
    crud_repository::find_with_sort(
        doc! {"riderId": rider_id },
        doc! {"year": 1, "month": 1},
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id_year(
    rider_id: &str,
    year: i32,
    db: &Database,
) -> Result<Vec<RiderMonthlySummary>, Error> {
    crud_repository::find_with_sort(
        doc! {"riderId": rider_id, "year": year  },
        doc! {"month": 1},
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn increment_activity_stats(
    rider_id: &str,
    year: i32,
    month: i32,
    activity_type: &ActivityType,
    stats: &Stats,
    db: &Database,
) {
    log::trace!("increment_activity_stats");
    init_rider_summary(rider_id, year, month, db).await;

    if let Some(update_document) = helpers::get_activity_stats_update_document(
        activity_type,
        stats,
    ) {
        crud_repository::update_one(
            doc! {"riderId": rider_id, "year": year, "month": month},
            update_document,
            None,
            COLLECTION_NAME,
            db,
        )
        .await
        .unwrap();
    } else {
        log::trace!(
            "None from get_activity_stats_update_document for {:?}",
            &activity_type
        );
    }
}

async fn init_rider_summary(rider_id: &str, year: i32, month: i32, db: &Database) {
    let rider_summary = crud_repository::find_one::<Any>(
        doc! {"riderId": rider_id, "year": year, "month": month},
        COLLECTION_NAME,
        db,
    )
    .await
    .unwrap();

    if let None = rider_summary {
        let empty_rider_summary = RiderMonthlySummary::new(rider_id.into(), year, month);
        crud_repository::add(&empty_rider_summary, COLLECTION_NAME, db)
            .await
            .unwrap();
    }
}
